--
-- file:   string.e
-- author: Normand M. Blais
-- date:   January 4 1999
--
------------------------------<string.e>--------------------------------
-- type string
-- Definition: string =  sequence of words separeted by space.
-- Ex.: sequence s
--               s = "Hello World"  -- two words, one blank character 
--
global type string_type(object x)
   if sequence(x) then
      if length(x) = 0 then
         return 1 -- empty string
      else 
         for i=1 to length(x) do
            if not atom(x[i]) then -- not a character
               return 0
            elsif not (x[i] >= 32 and x[i] <= 126) then -- not printable
               return 0
            end if
         end for   
         return 1 -- all printable characters
      end if
   end if
   return 0
end type        

-------------------------------------------------------------------------
-----------------------------<trim_spaces>-------------------------------
global
function trim_spaces(string_type s)
-- returns the original string less the space characters at the 
-- beginning and at the end if any.
    string_type ss
    if length(s) = 0 then
        return s
    end if      
    ss = s
    for i=1 to length(s)-1 do
        if s[i] != ' ' then
            exit
        else
            ss = s[i+1..length(s)]
        end if
    end for 
    s = ss
    for i=length(ss) to 2 by -1 do
        if ss[i] != ' ' then
            exit
        else
            s = ss[1..i-1]
        end if
    end for
    return s
end function

-----------------------------<compress_spaces>-------------------------------
global
function compress_spaces(string_type s)
-- returns a trimed string with multiple space characters inside
-- the string reduced to one space characters between words.
    string_type ss
    if length(s) = 0 then
        return s
    end if      
    s = trim_spaces(s)
    ss = {}
    for i=1 to length(s)-1 do
        if s[i] = ' ' and s[i+1] = ' ' then
            -- continue
        else
            ss = ss & s[i]    
        end if
    end for               
    return ss & s[length(s)]
end function

-----------------------------<found_space>-------------------------------
-- local
function found_space(string_type s, integer i)
-- determines whether i points to a word boundary position
    if i > 1 and length(s) then
        return ( s[i-1] = ' ' and s[i] != ' ')
    else
        return 0
    end if
end function

-----------------------------<count_words>-------------------------------
global
function count_words(string_type s)
-- counts the numbers of words in a string
    integer count
    if length(s) = 0 then
        return 0
    end if  
    -- add space before and after the string
    s = ' '& s & ' '    
    count = 0
    -- count words by looking for spaces
    for i=1 to length(s) do
        if found_space(s,i) then
            count = count + 1
        end if
    end for                  
    return count
end function

-----------------------------<left_str>-------------------------------
global
function left_str(string_type s, integer numchars)
-- returns a string of "numchars" characters starting from
-- the left of the original string.
    if length(s) = 0 or numchars <= 0 then
        return {}
    end if
    return s[1..numchars]
end function

-----------------------------<right_str>-------------------------------
global
function right_str(string_type s, integer numchars)
-- returns a string of "numchars" characters starting from
-- the right of the original string.
    if length(s) = 0 or numchars <= 0 or numchars > length(s) then
        return {}
    end if
    return s[length(s)-numchars+1..length(s)]
end function

-----------------------------<mid_str>-------------------------------
global
function mid_str(string_type s, integer first, integer numchars)
-- returns a string of "numchars" starting from "first" to
-- "first+numchars-1" of the original string.
    if length(s) = 0 or first <= 0 or numchars <= 0 
        or first+numchars > length(s) then
        return {}
    end if
    return s[first..first+numchars-1]
end function


